/*
	RackAFX(TM)
	Applications Programming Interface
	Derived Class Object Definition
	Copyright(c) Tritone Systems Inc. 2006-2012

	Your plug-in must implement the constructor,
	destructor and virtual Plug-In API Functions below.
*/

#pragma once

// base class
#include "plugin.h"

// abstract base class for RackAFX filters
class CRotarySpeakerSim : public CPlugIn
{
public:
	// RackAFX Plug-In API Member Methods:
	// The followung 5 methods must be impelemented for a meaningful Plug-In
	//
	// 1. One Time Initialization
	CRotarySpeakerSim();

	// 2. One Time Destruction
	virtual ~CRotarySpeakerSim(void);

	// 3. The Prepare For Play Function is called just before audio streams
	virtual bool __stdcall prepareForPlay();

	// 4. processAudioFrame() processes an audio input to create an audio output
	virtual bool __stdcall processAudioFrame(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);

	// 5. userInterfaceChange() occurs when the user moves a control.
	virtual bool __stdcall userInterfaceChange(int nControlIndex);


	// OPTIONAL ADVANCED METHODS ------------------------------------------------------------------------------------------------
	// These are more advanced; see the website for more details
	//
	// 6. initialize() is called once just after creation; if you need to use Plug-In -> Host methods
	//				   such as sendUpdateGUI(), you must do them here and NOT in the constructor
	virtual bool __stdcall initialize();

	// 7. joystickControlChange() occurs when the user moves a control.
	virtual bool __stdcall joystickControlChange(float fControlA, float fControlB, float fControlC, float fControlD, float fACMix, float fBDMix);

	// 8. process buffers instead of Frames:
	// NOTE: set m_bWantBuffers = true to use this function
	virtual bool __stdcall processRackAFXAudioBuffer(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels, UINT uBufferSize);

	// 9. rocess buffers instead of Frames:
	// NOTE: set m_bWantVSTBuffers = true to use this function
	virtual bool __stdcall processVSTAudioBuffer(float** inBuffer, float** outBuffer, UINT uNumChannels, int inFramesToProcess);

	// 10. MIDI Note On Event
	virtual bool __stdcall midiNoteOn(UINT uChannel, UINT uMIDINote, UINT uVelocity);

	// 11. MIDI Note Off Event
	virtual bool __stdcall midiNoteOff(UINT uChannel, UINT uMIDINote, UINT uVelocity, bool bAllNotesOff);


	// 12. MIDI Modulation Wheel uModValue = 0 -> 127
	virtual bool __stdcall midiModWheel(UINT uChannel, UINT uModValue);

	// 13. MIDI Pitch Bend
	//					nActualPitchBendValue = -8192 -> 8191, 0 is center, corresponding to the 14-bit MIDI value
	//					fNormalizedPitchBendValue = -1.0 -> +1.0, 0 is at center by using only -8191 -> +8191
	virtual bool __stdcall midiPitchBend(UINT uChannel, int nActualPitchBendValue, float fNormalizedPitchBendValue);

	// 14. MIDI Timing Clock (Sunk to BPM) function called once per clock
	virtual bool __stdcall midiClock();


	// 15. all MIDI messages -
	// NOTE: set m_bWantAllMIDIMessages true to get everything else (other than note on/off)
	virtual bool __stdcall midiMessage(unsigned char cChannel, unsigned char cStatus, unsigned char cData1, unsigned char cData2);

	// 16. initUI() is called only once from the constructor; you do not need to write or call it. Do NOT modify this function
	virtual bool __stdcall initUI();



	// Add your code here: ----------------------------------------------------------- //

	// Distortion
	float mfK;
	float mf1_ArcTanK;
	float mfMaxK;

	// Horn Angle
	float mfTheta;  // Current horn angle
	float mfThetaInc;  // amount in increment theta each sample
	float mfSpdSound; // 1132 ft/sec (345 m/s)
	int mdBufferSize; // 0.0075 seconds * fs
	float mfDelayL;  // left channel delay in samples
	float mfDelayR;  // right channel delay in samples
	float mfAmplL, mfAmplR;  // Amplitude Modulation Gain (dB)
	float mfGainL, mfGainR; // Amplitude Mod Raw Gain
	void updateTheta(); // increment theta, calc new delay and fc values

	// Delay
	float *mpDelayBuf;   // main delay buffer
	int mdWriteIdx;
	float mfReadIdxL, mfReadIdxR;
	float mfDelOutL, mfDelOutR;
	void getDelOutputs();

	// Modulated LPF
	CBiQuad mLPFLeft;
	CBiQuad mLPFRight;
	float mfFilterQ;
	void calcCoeffs(float fcLeft, float fcRight);

	// Horn Resonance
	CBiQuad mHornResL;
	CBiQuad mHornResR;
	float mfHornQ;
	void calcHornCoeffs();

	// Crossovers
	CBiQuad mHornHPF;
	CBiQuad mBassLPF;
	CBiQuad mBassHPF;
	CBiQuad mBypassLPF;
	void crossCoeffs(float fc, bool bMainCross);

	// Bass Modulation
	float mfBassTheta;	// Current angle of bass rotor
	float mfBassThetaInc;
	float mfGainB;   // AM gain applied to bass signal
	void updateBassTheta();

	float mfTrebGain;
	float mfBassGain;

	// Speed Control - Treble
	bool bUseAutoSpeed, bSpeedChange, bSpeedAccel;
	float mfOffSpeed, mfSlowSpeed, mfFastSpeed;  // fixed values for auto speed control
	float mfCurSpeed, mfSpeedTarget;
	float mfSpeedInc, mfSpeedDec; // increment and decrement values of speed (changed by accel and brake)
	float mfHornRad1, mfSlowHornRad, mfFastHornRad;
	float mfHornRPMtoRadConv;  // RPM to horn radius conversion factor 

	// Speed Control - Bass
	bool bSpeedChangeB, bSpeedAccelB;
	float mfOffSpeedB, mfSlowSpeedB, mfFastSpeedB;  // fixed values for auto speed control
	float mfCurSpeedB, mfSpeedTargetB;
	float mfSpeedIncB, mfSpeedDecB; // increment and decrement values of speed (changed by accel and brake)


	// END OF USER CODE -------------------------------------------------------------- //


	// ADDED BY RACKAFX -- DO NOT EDIT THIS CODE!!! ----------------------------------- //
	//  **--0x07FD--**

	float mfTrebLPFModDepth;
	float mfTrebLPFfc;
	float mfHornRad;
	float mfDistLevel;
	float mfRotSpd;
	float mfTrebleAtten;
	float mfHornResF;
	float mfHornResDB;
	float mfAmplMod;
	float mfAccel;
	float mfBrake;
	float mfBassSpd;
	float mfBassMod;
	float mfBassAccel;
	float mfBassBrake;
	float mfBassXover;
	float mfBassAtten;
	UINT muSpeedControl;
	enum{MAN,AUTO};
	UINT muAutoSpeed;
	enum{OFF,SLOW,FAST};

	// **--0x1A7F--**
	// ------------------------------------------------------------------------------- //

};




